/*

 eth.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "eth.h"
#ifdef _ETH_H
#include <string.h>
#ifdef HAVE_WS2
#include <stdlib.h>
#include "ws2.h"
#include "iphlpapil.h"
#include "message.h"
#include "message.h"
#define socklen_t int
#else
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <net/if.h>
#ifndef SIOCGIFADDR
#include <ifaddrs.h>
#endif
#ifndef IP_PKTINFO
#ifdef IP_RECVIF
#include <net/if_dl.h>
#endif
#endif
#endif
#include <stdio.h>

#include "log.h"
#include "driver.h"

#ifndef MSG_NOSIGNAL
#define MSG_NOSIGNAL 0
#endif

#define SERVER_IP "127.0.0.1"
#define SERVER_PORT 64000
#define MULTICAST_ADDR 0xef004000

static unsigned char ebufi[1030], ebufo[1030];
static unsigned int ebufip, ebufop, ebufil;
#ifdef WSAAPI
static int wsainited;
#ifdef _WS2_H
static int ws2_loaded;
#endif
#ifdef _IPHLPAPIL_H
static int iphlpapi_loaded;
#endif
#ifndef WSAID_WSARECVMSG
#define WSAID_WSARECVMSG {0xf689d7c8,0x6f1f,0x436b,{0x8a,0x53,0xe5,0x4f,0xe3,0x51,0xc3,0x22}}
#endif
#if defined WSA_CMSG_FIRSTHDR || defined WSA_CMSG_NXTHDR || defined WSA_CMSG_DATA
#else
#define WSA_CMSG_FIRSTHDR(msg) (((msg)->Control.len >= sizeof(WSACMSGHDR)) ? (WSACMSGHDR *)(msg)->Control.buf : (WSACMSGHDR *)NULL)
#define WSA_CMSG_NXTHDR(msg,cmsg) ((!(cmsg)) ? WSA_CMSG_FIRSTHDR(msg) : ((((unsigned char *)(cmsg) + (((cmsg)->cmsg_len + 3) & ~3) + sizeof(WSACMSGHDR)) > (unsigned char *)((msg)->Control.buf) + (msg)->Control.len) ? (WSACMSGHDR *)NULL : (WSACMSGHDR *)((unsigned char *)(cmsg) + (((cmsg)->cmsg_len + 3) & ~3))))
#define WSA_CMSG_DATA(cmsg) ((unsigned char *)(cmsg) + ((sizeof(WSACMSGHDR) + 3) & ~3))
#endif
static int (*WSARecvMsg_)(SOCKET, LPWSAMSG, LPDWORD, LPWSAOVERLAPPED, LPWSAOVERLAPPED_COMPLETION_ROUTINE);
static SOCKET sock = INVALID_SOCKET;
static WSABUF wsabuf;
static WSAOVERLAPPED overlapped;
static UINT server_if_cache;
#else
#define closesocket(s) close(s)
#define INVALID_SOCKET (-1)
#define SOCKET_ERROR (-1)
static int sock = INVALID_SOCKET;
static int server_if_cache;
#endif
static struct in_addr server_addr_cache;
static struct sockaddr_in client_addr;
static const char *i_addr;
static unsigned int i_network;
static int inited;
static Driver_errno driver_errno = DRIVER_EIO;

#ifdef WSAAPI
#define get_error() WSAGetLastError()
#else
#define get_error() (errno)
#define errormsg(err) strerror(err)
#endif

static void error_print(const char *txt) {
    int error = get_error();
    log_printf(txt, errormsg(error), error);
}

#ifdef WSAAPI
static int ifindex_to_addr(UINT ifindex, struct in_addr *addr) {
    PMIB_IPADDRTABLE ipaddrtable;
    DWORD size = 0;
    DWORD ret = 0;

    ipaddrtable = (MIB_IPADDRTABLE *)malloc(sizeof *ipaddrtable);
    if (ipaddrtable == NULL) {
        return -1;
    }
    ret = GetIpAddrTable(ipaddrtable, &size, 0);
    if (ret == ERROR_INSUFFICIENT_BUFFER) {
        free(ipaddrtable);
        ipaddrtable = (MIB_IPADDRTABLE *)malloc(size);
        if (ipaddrtable == NULL) {
            return -1;
        }
        ret = GetIpAddrTable(ipaddrtable, &size, 0);
    }
    if (ret == NO_ERROR) { 
        unsigned int i;
        for (i = 0; i < ipaddrtable->dwNumEntries; i++) {
            if (ipaddrtable->table[i].dwIndex == ifindex) {
                addr->s_addr = ipaddrtable->table[i].dwAddr;
                free(ipaddrtable);
                return 0;
            }
        }
    }
    free(ipaddrtable);
    return -1;
}
#elif defined SIOCGIFADDR
static int ifindex_to_addr(int ifindex, struct in_addr *addr) {
    struct ifreq ifr;
#ifdef SIOCGIFNAME
    ifr.ifr_ifindex = ifindex;
    if (ioctl(sock, SIOCGIFNAME, &ifr) < 0) return -1;
#else
    if (if_indextoname(ifindex, ifr.ifr_name) == NULL) {
        return -1;
    }
#endif
    if (ioctl(sock, SIOCGIFADDR, &ifr) < 0) return -1;
    memcpy(addr, &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr, sizeof *addr);
    return 0;
}
#else
static int ifindex_to_addr(int ifindex, struct in_addr *addr) {
    struct ifaddrs *ifaddr;
    struct ifaddrs *p;
    char ifname[IF_NAMESIZE];

    if (if_indextoname(ifindex, ifname) == NULL) {
        return -1;
    }

    if (getifaddrs(&ifaddr) == -1) {
        return -1;
    }

    for (p = ifaddr; p != NULL; p = p->ifa_next) {
        if (p->ifa_addr == NULL) continue;
        if (p->ifa_addr->sa_family != AF_INET) continue;
        if (strcmp(p->ifa_name, ifname) != 0) continue;
        memcpy(addr, &((struct sockaddr_in *)p->ifa_addr)->sin_addr, sizeof *addr);
        freeifaddrs(ifaddr);
        return 0;
    }
    freeifaddrs(ifaddr);
    return -1;
}
#endif

static int initialize(void) {
    struct sockaddr_in addr;
    static struct hostent *hp;

    driver_errno = DRIVER_EIO;
    inited = 0;
    server_if_cache = 0;
    client_addr.sin_family = AF_INET;
    client_addr.sin_addr.s_addr = htonl(MULTICAST_ADDR + i_network);
    client_addr.sin_port = htons((unsigned short)(SERVER_PORT + i_network));

#ifdef WSAAPI
#ifdef _WS2_H
    if (!ws2_loaded) {
        log_print("Winsock 2 not found");
        return 1;
    }
#endif
#ifdef _IPHLPAPIL_H
    if (!iphlpapi_loaded) {
        log_print("IPHLPAPI not found");
        return 1;
    }
#endif

    if (wsainited == 0) {
        WORD versionWanted = MAKEWORD(1, 1);
        WSADATA wsaData;
        int wsaerrno = WSAStartup(versionWanted, &wsaData);
        if (wsaerrno != 0) {
            log_printf("Cannot start winsock: %s(%d)", errormsg(wsaerrno), wsaerrno);
            return 1;
        }
        wsainited = 1;
    }
#endif

    if (i_addr != NULL) {
        if (hp == NULL) {
            hp = gethostbyname(i_addr);
            if (hp == NULL) {
#ifdef WSAAPI
                int error = get_error();
                log_printf("Can't resolve address of %s: %s(%d)", i_addr, errormsg(error), error);
#else
                log_printf("Can't resolve address of %s: %s(%d)", i_addr, hstrerror(h_errno), h_errno);
#endif
                return 1;
            }
        }
        if (hp->h_addrtype != AF_INET || hp->h_addr == NULL || hp->h_length != sizeof server_addr_cache.s_addr) {
            log_printf("Can't resolve address of %s", i_addr);
            return 1;
        }
        memcpy(&server_addr_cache.s_addr, hp->h_addr, sizeof server_addr_cache.s_addr);
    } else {
        server_addr_cache.s_addr = INADDR_ANY;
    }

    if (sock == INVALID_SOCKET) {
        sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    }
    if (sock == INVALID_SOCKET) {
        error_print("Cannot open socket: %s(%d)");
        return 1;
    }
#ifdef WSAAPI
    if (overlapped.hEvent == WSA_INVALID_EVENT) {
        memset(&overlapped, 0, sizeof overlapped);
        overlapped.hEvent = WSACreateEvent();
        if (overlapped.hEvent == WSA_INVALID_EVENT) {
            int error = get_error();
            log_printf("WSACreateEvent failed: %s(%d)", errormsg(error), error);
            return 1;
        }
    }

    if (WSARecvMsg_ == NULL) {
        GUID g = WSAID_WSARECVMSG;
        DWORD ret = 0;
        if (WSAIoctl(sock, SIO_GET_EXTENSION_FUNCTION_POINTER, &g, sizeof g, &WSARecvMsg_, sizeof WSARecvMsg_, &ret, NULL, NULL) != 0) {
            WSARecvMsg_ = NULL;
            if (i_addr == NULL) {
                error_print("WSARecvMsg not available: %s(%d)");
                log_print("Use the -i option to set the interface address");
                return 1;
            }
        }
    }
#endif

#ifdef IP_PKTINFO
    {
#ifdef WSAAPI
        DWORD tr = 1;
#else
        int tr = 1;
#endif
        if (setsockopt(sock, IPPROTO_IP, IP_PKTINFO, (char *)&tr, sizeof tr) == SOCKET_ERROR) {
            error_print("Setting socket options failed: %s(%d)");
            return 1;
        }
    }
#else
#ifdef IP_RECVIF
    {
        int tr = 1;
        if (setsockopt(sock, IPPROTO_IP, IP_RECVIF, &tr, sizeof tr) == SOCKET_ERROR) {
            error_print("Setting socket options failed: %s(%d)");
            return 1;
        }
    }
#endif
#ifdef IP_RCVDSTADDR
    {
        int tr = 1;
        if (setsockopt(sock, IPPROTO_IP, IP_RCVDSTADDR, &tr, sizeof tr) == SOCKET_ERROR) {
            error_print("Setting socket options failed: %s(%d)");
            return 1;
        }
    }
#endif
#endif

    {
#ifdef WSAAPI
        BOOL tr = 1;
#else
        int tr = 1;
#endif
        if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&tr, sizeof tr) == SOCKET_ERROR) {
            error_print("Setting socket options failed: %s(%d)");
            return 1;
        }
    }
    
    memset(&addr, 0, sizeof addr);
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = INADDR_ANY;
    addr.sin_port = htons((unsigned short)(SERVER_PORT + i_network));
    if (bind(sock, (struct sockaddr *)&addr, sizeof addr) == SOCKET_ERROR) {
        int error = get_error();
        log_printf("Cannot bind to port %d: %s(%d)", SERVER_PORT + i_network, errormsg(error), error);
        return 1;
    }

    if (i_addr != NULL) {
        if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, (char *)hp->h_addr, (socklen_t)hp->h_length) == SOCKET_ERROR) {
            error_print("Setting multicast interface failed: %s(%d)");
            log_printf("Assuming %s is the C64's address with static ARP using an old IDEDOS.", i_addr);
            log_print("With a new IDEDOS it should be corrected to the network interface address.");
            memcpy(&client_addr.sin_addr.s_addr, hp->h_addr, sizeof client_addr.sin_addr.s_addr);
            server_addr_cache.s_addr = INADDR_ANY;
        }
    }

    if (i_addr == NULL || server_addr_cache.s_addr != INADDR_ANY) {
        log_printf("Listening on %s:%d for C64", i_addr != NULL ? i_addr : "0.0.0.0", SERVER_PORT + i_network);
    } else {
        log_printf("Listening on port %d for C64 at %s", SERVER_PORT + i_network, i_addr);
    }

#ifdef WSAAPI
    memset(&wsabuf, 0, sizeof wsabuf);
    wsabuf.buf = (char *)ebufi + 2;
    wsabuf.len = sizeof ebufi  - 2;
#endif
    inited = 1;
    return 0;
}

static int getb(int use_timeout) {
    (void)use_timeout;
    if (driver_errno != DRIVER_SUCCESS) return EOF;
    if (ebufip >= sizeof ebufi) return EOF;
    if (ebufip >= ebufil) return EOF;
    return ebufi[ebufip++];
}

static void sendb(unsigned char a) {
    if (ebufop >= sizeof ebufo) {
        if (driver_errno != DRIVER_SUCCESS) return;
        driver_errno = DRIVER_EIO;
        return;
    }
    ebufo[ebufop++] = a;
}

static void getbytes(unsigned char data[], unsigned int bytes) {
    if (ebufip + bytes > ebufil) {
        memcpy(data, ebufi + ebufip, ebufil - ebufip);
        ebufip = ebufil;
        if (driver_errno != DRIVER_SUCCESS) return;
        driver_errno = DRIVER_EIO;
        return;
    }
    memcpy(data, ebufi + ebufip, bytes);
    ebufip += bytes;
}

static void sendbytes(const unsigned char data[], unsigned int bytes) {
    if (ebufop + bytes > sizeof ebufo) {
        memcpy(ebufo + ebufop, data, sizeof ebufo - ebufop);
        ebufop = sizeof ebufo;
        if (driver_errno != DRIVER_SUCCESS) return;
        driver_errno = DRIVER_EIO;
        return;
    }
    memcpy(ebufo + ebufop, data, bytes);
    ebufop += bytes;
}

static void eshutdown(void) {
    if (sock != INVALID_SOCKET) {
        closesocket(sock);
        sock = INVALID_SOCKET;
    }
#ifdef WSAAPI
    if (overlapped.hEvent != WSA_INVALID_EVENT) {
        WSACloseEvent(overlapped.hEvent);
        overlapped.hEvent = WSA_INVALID_EVENT;
    }
    if (wsainited == 1) {
        WSACleanup();
        wsainited = 0;
    }
#endif
    inited = 0;
    return;
}

static Driver_errno flush(void) {
    if (ebufop != 0 && driver_errno == DRIVER_SUCCESS) {
        if (sendto(sock, (char *)ebufo, ebufop, MSG_NOSIGNAL, (struct sockaddr *)&client_addr, sizeof client_addr) == SOCKET_ERROR) {
            error_print("Couldn't send telegram: %s(%d)");
            ebufop = 0;
            return DRIVER_EIO;
        }
        ebufop = 0;
    }
    return driver_errno;
}

static Driver_errno done(void) {
    return driver_errno;
}

static void turn(void) {
}

static int clean(void) {
    ebufip = 0;
    ebufil = 0;
    ebufop = 0;
    return 0;
}

static int waitb(unsigned char ec) {
    struct sockaddr_in addr;
    int n;
    (void)ec;
    if (!inited) return DRIVER_ENODEV;
    ebufip = ebufil = 0;
#ifdef CMSG_FIRSTHDR
    do {
        struct msghdr msg;
        struct iovec iov;
        char cbuf[256]; 
        struct cmsghdr *p;

        memset(&iov, 0, sizeof iov);
        iov.iov_base = (char *)ebufi + 2;
        iov.iov_len = sizeof ebufi - 2;
        memset(&msg, 0, sizeof msg);
        msg.msg_name = &addr;
        msg.msg_namelen = sizeof addr;
        msg.msg_iov = &iov;
        msg.msg_iovlen = 1;
        msg.msg_control = &cbuf;
        msg.msg_controllen = sizeof cbuf;
        msg.msg_flags = 0;
        n = recvmsg(sock, &msg, 0);
        if (n == SOCKET_ERROR || n < 0) return DRIVER_EIO;
        if ((msg.msg_flags & MSG_CTRUNC) != 0) return DRIVER_EIO;
        if ((msg.msg_flags & MSG_TRUNC) != 0) continue;
        if (addr.sin_family != AF_INET) continue;
        if (addr.sin_addr.s_addr != INADDR_ANY) continue;

        for (p = CMSG_FIRSTHDR(&msg); p != NULL; p = CMSG_NXTHDR(&msg, p)) {
            int ifindex = server_if_cache;
            if (p->cmsg_level != IPPROTO_IP) continue;
#ifdef IP_PKTINFO
            if (p->cmsg_type != IP_PKTINFO) continue;
            if (((struct in_pktinfo *)CMSG_DATA(p))->ipi_addr.s_addr != INADDR_BROADCAST) break;
            ifindex = ((struct in_pktinfo *)CMSG_DATA(p))->ipi_ifindex;
#else 
#ifdef IP_RCVDSTADDR
            if (p->cmsg_type == IP_RCVDSTADDR) {
                if (((struct in_addr *)CMSG_DATA(p))->s_addr != INADDR_BROADCAST) break;
                continue;
            }
#endif
#ifdef IP_RECVIF
            if (p->cmsg_type != IP_RECVIF) continue;
            ifindex = ((struct sockaddr_dl *)CMSG_DATA(p))->sdl_index;
#endif
#endif
            if (server_if_cache != ifindex) {
                struct in_addr addr2;
                if (ifindex_to_addr(ifindex, &addr2) == 0) {
                    if (server_addr_cache.s_addr != INADDR_ANY) {
                        if (server_addr_cache.s_addr != addr2.s_addr) break;
                    } else {
                        if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, &addr2, sizeof addr2) == SOCKET_ERROR) {
                            error_print("Setting multicast interface failed: %s(%d)");
                            return DRIVER_EIO;
                        }
                    }
                    server_if_cache = ifindex;
                } 
            }
        }
        if (p == NULL) break;
    } while (1);
#elif defined WSAAPI
    do {
        if (WSARecvMsg_ != NULL) {
            WSAMSG msg;
            DWORD len;
            char cbuf[256]; 
            WSACMSGHDR *p;
            int ret;

            memset(&msg, 0, sizeof msg);
            msg.name = (struct sockaddr *)&addr;
            msg.namelen = sizeof addr;
            msg.lpBuffers = &wsabuf;
            msg.dwBufferCount = 1;
            msg.Control.buf = cbuf;
            msg.Control.len = sizeof cbuf;
            msg.dwFlags = 0;
            WSAResetEvent(overlapped.hEvent);
            ret = WSARecvMsg_(sock, &msg, &len, &overlapped, NULL);
            if (ret == SOCKET_ERROR) { 
                ret = WSAGetLastError();
                if (ret == WSAEMSGSIZE || ret == WSAEINPROGRESS) continue;
                if (ret != WSA_IO_PENDING) return DRIVER_EIO;
                if (WSAWaitForMultipleEvents(1, &overlapped.hEvent, TRUE, INFINITE, TRUE) == WSA_WAIT_FAILED) return DRIVER_EIO;
                if (WSAGetOverlappedResult(sock, &overlapped, &len, FALSE, &msg.dwFlags) == FALSE) return DRIVER_EIO;
            } else if (ret != 0) return DRIVER_EIO;
            if (addr.sin_family != AF_INET) continue;
            if (addr.sin_addr.s_addr != INADDR_ANY) continue;

            for (p = WSA_CMSG_FIRSTHDR(&msg); p != NULL; p = WSA_CMSG_NXTHDR(&msg, p)) {
                struct in_pktinfo *pi;
                if (p->cmsg_type != IP_PKTINFO) continue;
                pi = (struct in_pktinfo *)WSA_CMSG_DATA(p);
                if (pi->ipi_addr.s_addr != INADDR_BROADCAST) break;
                if (server_if_cache != pi->ipi_ifindex) {
                    struct in_addr addr2;
                    if (ifindex_to_addr(pi->ipi_ifindex, &addr2) == 0) {
                        if (server_addr_cache.s_addr != INADDR_ANY) {
                            if (server_addr_cache.s_addr != addr2.s_addr) break;
                        } else {
                            if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF, (char *)&addr2, sizeof addr2) == SOCKET_ERROR) {
                                error_print("Setting multicast interface failed: %s(%d)");
                                return DRIVER_EIO;
                            }
                        }
                        server_if_cache = pi->ipi_ifindex;
                    }
                }
            }
            n = len;
            if (p == NULL) break;
        } else {
            socklen_t addrlen = sizeof addr;
            n = recvfrom(sock, (char *)ebufi + 2, sizeof ebufi - 2, MSG_NOSIGNAL, (struct sockaddr *)&addr, &addrlen);
            if (n == SOCKET_ERROR || n < 0) return DRIVER_EIO;
            if (addr.sin_family == AF_INET && addr.sin_addr.s_addr == INADDR_ANY) break;
        }
    } while (1);
#else
    do {
        socklen_t addrlen = sizeof addr;
        n = recvfrom(sock, (char *)ebufi + 2, sizeof ebufi - 2, MSG_NOSIGNAL, (struct sockaddr *)&addr, &addrlen);
        if (n == SOCKET_ERROR || n < 0) return DRIVER_EIO;
    } while (addr.sin_family != AF_INET || addr.sin_addr.s_addr != INADDR_ANY);
#endif
    ebufi[0] = (unsigned char)(ntohs(addr.sin_port) >> 8);
    ebufi[1] = (unsigned char)ntohs(addr.sin_port);
    driver_errno = DRIVER_SUCCESS;
    ebufip = ebufi[0] ? 0 : 2;
    ebufil = (unsigned int)n + 2;
    return ebufi[ebufip++];
}

static void destroy(void) { 
#ifdef _WS2_H
    if (ws2_loaded) {
        ws2_deinit();
        ws2_loaded = 0;
    }
#endif
#ifdef _IPHLPAPIL_H
    if (iphlpapi_loaded) {
        iphlpapi_deinit();
        iphlpapi_loaded = 0;
    }
#endif
}

static const Driver driver = {
    .mode         = M_ETHERNET,
    .name         = "ETH",
    .initialize   = initialize,
    .getb         = getb,
    .sendb        = sendb,
    .getbytes     = getbytes,
    .sendbytes    = sendbytes,
    .shutdown     = eshutdown,
    .flush        = flush,
    .done         = done,
    .turn         = turn,
    .wait         = waitb,
    .clean        = clean,
    .destroy      = destroy,
};

const Driver *eth_driver(const char *addr, int network) {
    i_addr = addr;
    i_network = (unsigned char)network;
#ifdef _WS2_H
    if (!ws2_loaded) {
        ws2_loaded = ws2_init();
    }
#endif
#ifdef _IPHLPAPIL_H
    if (!iphlpapi_loaded) {
        iphlpapi_loaded = iphlpapi_init();
    }
#endif
    log_printf("Using %s driver connecting from %s network %u", driver.name, i_addr != NULL ? i_addr : "0.0.0.0", i_network);
    return &driver;
}
#endif
