/*

 ftd2xxl.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>
 Win32 port by
  Josef Soucek <josef.soucek@ide64.org>
 Serial support by
  Borsanyi Attila <bigfoot@axelero.hu>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "ftd2xxl.h"
#ifdef _FTD2XXL_H
#include "log.h"
#include "message.h"

lpFT_CreateDeviceInfoList FT_CreateDeviceInfoList;
lpFT_GetDeviceInfoList FT_GetDeviceInfoList;
lpFT_OpenEx FT_OpenEx;
lpFT_ResetDevice FT_ResetDevice;
lpFT_Purge FT_Purge;
lpFT_SetTimeouts FT_SetTimeouts;
lpFT_SetLatencyTimer FT_SetLatencyTimer;
lpFT_Read FT_Read;
lpFT_Close FT_Close;
lpFT_SetChars FT_SetChars;
lpFT_Write FT_Write;

static HINSTANCE hLib;

static void *lookup(const char *name) {
    void *addr = GetProcAddress(hLib, name);
    if (addr == NULL) { 
        DWORD error = GetLastError();
        log_printf("GetProcAddress for %s failed: %s(%u)", name, errormsg(error), (unsigned int)error); 
    }
    return addr;
}

int ftd2xx_init(void) {
    if (hLib == NULL) {
        hLib = LoadLibrary("ftd2xx.dll");
        if (hLib == NULL) { 
            DWORD error = GetLastError();
            log_printf("LoadLibrary of ftd2xx.dll failed: %s(%u)", errormsg(error), (unsigned int)error); 
            return 0; 
        }
    }
    FT_CreateDeviceInfoList = lookup("FT_CreateDeviceInfoList");
    if (FT_CreateDeviceInfoList == NULL) return 0;
    FT_GetDeviceInfoList = (lpFT_GetDeviceInfoList)lookup("FT_GetDeviceInfoList");
    if (FT_GetDeviceInfoList == NULL) return 0;
    FT_OpenEx = (lpFT_OpenEx)lookup("FT_OpenEx");
    if (FT_OpenEx == NULL) return 0;
    FT_ResetDevice = (lpFT_ResetDevice)lookup("FT_ResetDevice");
    if (FT_ResetDevice == NULL) return 0;
    FT_Purge = (lpFT_Purge)lookup("FT_Purge");
    if (FT_Purge == NULL) return 0;
    FT_SetTimeouts = (lpFT_SetTimeouts)lookup("FT_SetTimeouts");
    if (FT_SetTimeouts == NULL) return 0;
    FT_SetLatencyTimer = (lpFT_SetLatencyTimer)lookup("FT_SetLatencyTimer");
    if (FT_SetLatencyTimer == NULL) return 0;
    FT_Read = (lpFT_Read)lookup("FT_Read");
    if (FT_Read == NULL) return 0;
    FT_Close = (lpFT_Close)lookup("FT_Close");
    if (FT_Close == NULL) return 0;
    FT_SetChars = (lpFT_SetChars)lookup("FT_SetChars");
    if (FT_SetChars == NULL) return 0;
    FT_Write = (lpFT_Write)lookup("FT_Write");
    if (FT_Write == NULL) return 0;
    return 1;
}

void ftd2xx_deinit(void) {
    if (hLib != NULL) {
        FreeLibrary(hLib);
        hLib = NULL;
    }
    FT_CreateDeviceInfoList = NULL;
    FT_GetDeviceInfoList = NULL;
    FT_OpenEx = NULL;
    FT_ResetDevice = NULL;
    FT_Purge = NULL;
    FT_SetTimeouts = NULL;
    FT_SetLatencyTimer = NULL;
    FT_Read = NULL;
    FT_Close = NULL;
    FT_SetChars = NULL;
    FT_Write = NULL;
}
#endif
