/*

 ftd2xxl.h - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>
 Win32 port by
  Josef Soucek <josef.soucek@ide64.org>
 Serial support by
  Borsanyi Attila <bigfoot@axelero.hu>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#if defined _WIN32 || defined __CYGWIN__
#ifndef _FTD2XXL_H
#define _FTD2XXL_H
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>

#define FTD2XX_API

typedef PVOID FT_HANDLE;
typedef ULONG FT_STATUS;

enum {
    FT_OK, FT_INVALID_HANDLE, FT_DEVICE_NOT_FOUND, FT_DEVICE_NOT_OPENED,
    FT_IO_ERROR, FT_INSUFFICIENT_RESOURCES, FT_INVALID_PARAMETER,
    FT_INVALID_BAUD_RATE, FT_DEVICE_NOT_OPENED_FOR_ERASE,
    FT_DEVICE_NOT_OPENED_FOR_WRITE, FT_FAILED_TO_WRITE_DEVICE,
    FT_EEPROM_READ_FAILED, FT_EEPROM_WRITE_FAILED, FT_EEPROM_ERASE_FAILED,
    FT_EEPROM_NOT_PRESENT, FT_EEPROM_NOT_PROGRAMMED, FT_INVALID_ARGS,
    FT_NOT_SUPPORTED, FT_OTHER_ERROR
};

enum {
    FT_OPEN_BY_SERIAL_NUMBER = 1, FT_OPEN_BY_DESCRIPTION = 2,
    FT_OPEN_BY_LOCATION = 4
};

enum {
    FT_PURGE_RX = 1, FT_PURGE_TX = 2
};

typedef struct {
    ULONG Flags;
    ULONG Type;
    ULONG ID;
    DWORD LocId;
    char SerialNumber[16];
    char Description[64];
    FT_HANDLE ftHandle;
} FT_DEVICE_LIST_INFO_NODE;

typedef FT_STATUS (WINAPI *lpFT_CreateDeviceInfoList)(LPDWORD);
typedef FT_STATUS (WINAPI *lpFT_GetDeviceInfoList)(FT_DEVICE_LIST_INFO_NODE *, LPDWORD);
typedef FT_STATUS (WINAPI *lpFT_OpenEx)(PVOID, DWORD, FT_HANDLE *);
typedef FT_STATUS (WINAPI *lpFT_ResetDevice)(FT_HANDLE);
typedef FT_STATUS (WINAPI *lpFT_Purge)(FT_HANDLE, ULONG);
typedef FT_STATUS (WINAPI *lpFT_SetTimeouts)(FT_HANDLE, ULONG, ULONG);
typedef FT_STATUS (WINAPI *lpFT_SetLatencyTimer)(FT_HANDLE, UCHAR);
typedef FT_STATUS (WINAPI *lpFT_Read)(FT_HANDLE, LPVOID, DWORD, LPDWORD);
typedef FT_STATUS (WINAPI *lpFT_Close)(FT_HANDLE);
typedef FT_STATUS (WINAPI *lpFT_SetChars)(FT_HANDLE, UCHAR, UCHAR, UCHAR, UCHAR);
typedef FT_STATUS (WINAPI *lpFT_Write)(FT_HANDLE, LPVOID, DWORD, LPDWORD);

extern lpFT_CreateDeviceInfoList FT_CreateDeviceInfoList;
extern lpFT_GetDeviceInfoList FT_GetDeviceInfoList;
extern lpFT_OpenEx FT_OpenEx;
extern lpFT_ResetDevice FT_ResetDevice;
extern lpFT_Purge FT_Purge;
extern lpFT_SetTimeouts FT_SetTimeouts;
extern lpFT_SetLatencyTimer FT_SetLatencyTimer;
extern lpFT_Read FT_Read;
extern lpFT_Close FT_Close;
extern lpFT_SetChars FT_SetChars;
extern lpFT_Write FT_Write;

extern int ftd2xx_init(void);
extern void ftd2xx_deinit(void);
#endif
#endif
