/*

 ws2.c

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>
 Win32 port by
  Josef Soucek <josef.soucek@ide64.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "iphlpapil.h"
#ifdef _IPHLPAPIL_H
#include "log.h"
#include "message.h"

lpGetIpAddrTable GetIpAddrTable;

static HINSTANCE hLib;

static void *lookup(const char *name) {
    void *addr = GetProcAddress(hLib, name);
    if (addr == NULL) { 
        DWORD error = GetLastError();
        log_printf("GetProcAddress for %s failed: %s(%u)", name, errormsg(error), (unsigned int)error); 
    }
    return addr;
}

int iphlpapi_init(void) {
    if (hLib == NULL) {
        hLib = LoadLibrary("iphlpapi.dll");
        if (hLib == NULL) {
            DWORD error = GetLastError();
            log_printf("LoadLibrary of iphlpapi.dll failed: %s(%u)", errormsg(error), (unsigned int)error); 
            return 0; 
        }
    }
    GetIpAddrTable = lookup("GetIpAddrTable");
    if (GetIpAddrTable == NULL) return 0;
    return 1;
}

void iphlpapi_deinit(void) {
    if (hLib != NULL) {
        FreeLibrary(hLib);
        hLib = NULL;
    }
    GetIpAddrTable = NULL;
}
#endif
