/*

 ws2.h

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>
 Win32 port by
  Josef Soucek <josef.soucek@ide64.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef HAVE_WS2
#ifndef _WS2_H
#define _WS2_H
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>

#define WSADESCRIPTION_LEN 256
#define WSASYS_STATUS_LEN 128

typedef struct WSAData {
    WORD wVersion;
    WORD wHighVersion;
#ifdef _WIN64
    unsigned short iMaxSockets;
    unsigned short iMaxUdpDg;
    char *lpVendorInfo;
    char szDescription[WSADESCRIPTION_LEN+1];
    char szSystemStatus[WSASYS_STATUS_LEN+1];
#else
    char szDescription[WSADESCRIPTION_LEN+1];
    char szSystemStatus[WSASYS_STATUS_LEN+1];
    unsigned short iMaxSockets;
    unsigned short iMaxUdpDg;
    char *lpVendorInfo;
#endif
} WSADATA, *LPWSADATA;

struct sockaddr;

typedef struct in_addr {
  union {
    struct { unsigned char b1, b2, b3, b4; } b;
    struct { unsigned short w1, w2; } w;
    unsigned long l;
  } ul;
} IN_ADDR, *PIN_ADDR, *LPIN_ADDR;

#define s_addr ul.l

struct sockaddr_in {
    short sin_family;
    unsigned short sin_port;
    struct in_addr sin_addr;
    char sin_zero[8];
};

struct hostent {
    char *h_name;
    char **h_aliases;
    short h_addrtype;
    short h_length;
    char **h_addr_list;
};

typedef struct _WSABUF {
    unsigned long len;
    char *buf;
} WSABUF, *LPWSABUF;

typedef struct _WSAMSG {                        
    struct sockaddr *name;                              
    INT namelen;                                  
    LPWSABUF lpBuffers;
    DWORD dwBufferCount;
    WSABUF Control;
    DWORD dwFlags;
} WSAMSG, *PWSAMSG, *LPWSAMSG;

typedef struct _WSACMSGHDR {
    SIZE_T cmsg_len;
    INT cmsg_level;
    INT cmsg_type;
} WSACMSGHDR,*PWSACMSGHDR,*LPWSACMSGHDR;

typedef struct in_pktinfo {
    IN_ADDR ipi_addr;
    UINT ipi_ifindex;
} IN_PKTINFO;

#define h_addr h_addr_list[0]

typedef UINT_PTR SOCKET;
#define INVALID_SOCKET (SOCKET)(~0)
#define SOCKET_ERROR (-1)
#define WSAOVERLAPPED OVERLAPPED
#define LPWSAOVERLAPPED LPOVERLAPPED
#define LPWSAOVERLAPPED_COMPLETION_ROUTINE LPOVERLAPPED_COMPLETION_ROUTINE
#define WSA_IO_PENDING ERROR_IO_PENDING
#define WSA_WAIT_FAILED WAIT_FAILED
#define WSA_INFINITE INFINITE
#define WSA_WAIT_TIMEOUT WAIT_TIMEOUT
#define AF_INET 2
#define INADDR_LOOPBACK 0x7f000001
#define INADDR_BROADCAST 0xfffffffful
#define INADDR_ANY 0x00000000ul
#define SOCK_STREAM 1
#define SOCK_DGRAM 2
#define WSAEVENT HANDLE
#define WSA_INVALID_EVENT ((WSAEVENT)NULL)
#define IPPROTO_IP 0
#define IPPROTO_TCP 6
#define IPPROTO_UDP 17
#define TCP_NODELAY 0x0001
#define SOL_SOCKET 0xffff
#define SO_REUSEADDR 0x0004
#define SD_BOTH 0x02
#define IOC_WS2 0x08000000
#define IOC_INOUT 0xc0000000ul
#define _WSAIORW(x,y) (IOC_INOUT|(x)|(y))
#define SIO_GET_EXTENSION_FUNCTION_POINTER _WSAIORW(IOC_WS2,6)
#define IP_MULTICAST_IF 9
#define IP_PKTINFO 19

#define WSAAPI WINAPI

typedef int (WSAAPI *lpWSAStartup)(WORD, LPWSADATA);
typedef struct hostent *(WSAAPI *lpgethostbyname)(const char *);
typedef int (WSAAPI *lpWSAGetLastError)(void);
typedef unsigned long (WSAAPI *lphtonl)(unsigned long);
typedef unsigned short (WSAAPI *lphtons)(unsigned short);
typedef SOCKET (WSAAPI *lpsocket)(int, int, int);
typedef WSAEVENT (WSAAPI *lpWSACreateEvent)(void);
typedef int (WSAAPI *lpconnect)(SOCKET, const struct sockaddr *, int);
typedef int (WSAAPI *lpclosesocket)(SOCKET);
typedef int (WSAAPI *lpsetsockopt)(SOCKET, int, int, const char *, int);
typedef WINBOOL (WSAAPI *lpWSAResetEvent)(WSAEVENT);
typedef int (WSAAPI *lpWSARecv)(SOCKET, LPWSABUF, DWORD, LPDWORD, LPDWORD, LPWSAOVERLAPPED, LPWSAOVERLAPPED_COMPLETION_ROUTINE);
typedef DWORD (WSAAPI *lpWSAWaitForMultipleEvents)(DWORD, const WSAEVENT *, WINBOOL, DWORD, WINBOOL);
typedef WINBOOL (WSAAPI *lpWSAGetOverlappedResult)(SOCKET, LPWSAOVERLAPPED, LPDWORD, WINBOOL, LPDWORD);
typedef int (WSAAPI *lpshutdown)(SOCKET, int);
typedef int (WSAAPI *lpWSAIoctl)(SOCKET, DWORD, LPVOID, DWORD, LPVOID, DWORD, LPDWORD, LPWSAOVERLAPPED, LPWSAOVERLAPPED_COMPLETION_ROUTINE);
typedef int (WSAAPI *lpbind)(SOCKET, const struct sockaddr *, int);
typedef WINBOOL (WSAAPI *lpWSACloseEvent)(WSAEVENT);
typedef int (WSAAPI *lpWSACleanup)(void);
typedef int (WSAAPI *lpsendto)(SOCKET, const char *, int, int, const struct sockaddr *, int);
typedef int (WSAAPI *lprecvfrom)(SOCKET, char *, int, int, struct sockaddr *, int *);
typedef int (WSAAPI *lpsend)(SOCKET, const char *, int, int);
typedef unsigned short (WSAAPI *lpntohs)(unsigned short);

extern lpWSAStartup WSAStartup;
extern lpgethostbyname gethostbyname;
extern lpWSAGetLastError WSAGetLastError;
extern lphtonl htonl;
extern lphtons htons;
extern lpsocket socket;
extern lpWSACreateEvent WSACreateEvent;
extern lpconnect connect;
extern lpclosesocket closesocket;
extern lpsetsockopt setsockopt;
extern lpWSAResetEvent WSAResetEvent;
extern lpWSARecv WSARecv;
extern lpWSAWaitForMultipleEvents WSAWaitForMultipleEvents;
extern lpWSAGetOverlappedResult WSAGetOverlappedResult;
extern lpshutdown shutdown;
extern lpWSAIoctl WSAIoctl;
extern lpbind bind;
extern lpWSACloseEvent WSACloseEvent;
extern lpWSACleanup WSACleanup;
extern lpsendto sendto;
extern lprecvfrom recvfrom;
extern lpsend send;
extern lpntohs ntohs;

extern int ws2_init(void);
extern void ws2_deinit(void);
#endif
#endif
